"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from squadcast_sdk import errors, models, utils
from squadcast_sdk._hooks import HookContext
from squadcast_sdk.apitoken import APIToken
from squadcast_sdk.apitokens import APITokens
from squadcast_sdk.types import OptionalNullable, UNSET
from squadcast_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class Users(BaseSDK):
    api_tokens: APITokens
    api_token: APIToken

    def __init__(
        self, sdk_config: SDKConfiguration, parent_ref: Optional[object] = None
    ) -> None:
        BaseSDK.__init__(self, sdk_config, parent_ref=parent_ref)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.api_tokens = APITokens(self.sdk_configuration, parent_ref=self.parent_ref)
        self.api_token = APIToken(self.sdk_configuration, parent_ref=self.parent_ref)

    def get_all(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.V3UsersUserResponse]:
        r"""Get All Users

        Returns all the users of the organization.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `read` scope.

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request(
            method="GET",
            path="/v3/users",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_getAllUsers",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.UsersGetAllUsersResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def get_all_async(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.V3UsersUserResponse]:
        r"""Get All Users

        Returns all the users of the organization.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `read` scope.

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request_async(
            method="GET",
            path="/v3/users",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_getAllUsers",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.UsersGetAllUsersResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def add(
        self,
        *,
        email: str,
        role: models.V3UsersAddUserRequestRole,
        first_name: str,
        last_name: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3UsersUserResponse:
        r"""Add User

        Add user to the organization with given role if not exists. Returns the user object in response.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param email:
        :param role:
        :param first_name:
        :param last_name:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3UsersAddUserRequest(
            email=email,
            role=role,
            first_name=first_name,
            last_name=last_name,
        )

        req = self._build_request(
            method="POST",
            path="/v3/users",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.V3UsersAddUserRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_addUser",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.UsersAddUserResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def add_async(
        self,
        *,
        email: str,
        role: models.V3UsersAddUserRequestRole,
        first_name: str,
        last_name: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3UsersUserResponse:
        r"""Add User

        Add user to the organization with given role if not exists. Returns the user object in response.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param email:
        :param role:
        :param first_name:
        :param last_name:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3UsersAddUserRequest(
            email=email,
            role=role,
            first_name=first_name,
            last_name=last_name,
        )

        req = self._build_request_async(
            method="POST",
            path="/v3/users",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.V3UsersAddUserRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_addUser",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.UsersAddUserResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def update_org_level_permissions(
        self,
        *,
        data: Union[
            List[models.V3UsersUpdateUserAbilitiesRequestData],
            List[models.V3UsersUpdateUserAbilitiesRequestDataTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> str:
        r"""Update Org Level Permissions

        :param data:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3UsersUpdateUserAbilitiesRequest(
            data=utils.get_pydantic_model(
                data, List[models.V3UsersUpdateUserAbilitiesRequestData]
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v3/users/abilities",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.V3UsersUpdateUserAbilitiesRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_updateOrgLevelPermissions",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.UsersUpdateOrgLevelPermissionsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def update_org_level_permissions_async(
        self,
        *,
        data: Union[
            List[models.V3UsersUpdateUserAbilitiesRequestData],
            List[models.V3UsersUpdateUserAbilitiesRequestDataTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> str:
        r"""Update Org Level Permissions

        :param data:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3UsersUpdateUserAbilitiesRequest(
            data=utils.get_pydantic_model(
                data, List[models.V3UsersUpdateUserAbilitiesRequestData]
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v3/users/abilities",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.V3UsersUpdateUserAbilitiesRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_updateOrgLevelPermissions",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.UsersUpdateOrgLevelPermissionsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        user_id: str,
        swap_user_id: str,
        suppress_incidents: bool,
        reassign_incidents: bool,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.UsersDeleteUserData:
        r"""Delete User

        This API replaces the swap_user for all the entities in Squadcast with user_id provided and deletes the user.

        :param user_id:
        :param swap_user_id:
        :param suppress_incidents:
        :param reassign_incidents:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UsersDeleteUserRequest(
            user_id=user_id,
            swap_user_id=swap_user_id,
            suppress_incidents=suppress_incidents,
            reassign_incidents=reassign_incidents,
        )

        req = self._build_request(
            method="PUT",
            path="/v3/users/delete-user",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.UsersDeleteUserRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_deleteUser",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.UsersDeleteUserResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        user_id: str,
        swap_user_id: str,
        suppress_incidents: bool,
        reassign_incidents: bool,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.UsersDeleteUserData:
        r"""Delete User

        This API replaces the swap_user for all the entities in Squadcast with user_id provided and deletes the user.

        :param user_id:
        :param swap_user_id:
        :param suppress_incidents:
        :param reassign_incidents:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UsersDeleteUserRequest(
            user_id=user_id,
            swap_user_id=swap_user_id,
            suppress_incidents=suppress_incidents,
            reassign_incidents=reassign_incidents,
        )

        req = self._build_request_async(
            method="PUT",
            path="/v3/users/delete-user",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.UsersDeleteUserRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_deleteUser",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.UsersDeleteUserResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def get_roles(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.V3UsersUserRoleResponse]:
        r"""Get User Roles

        Returns all available user roles.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `read` scope.

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request(
            method="GET",
            path="/v3/users/roles",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_getUserRoles",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.UsersGetUserRolesResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def get_roles_async(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.V3UsersUserRoleResponse]:
        r"""Get User Roles

        Returns all available user roles.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `read` scope.

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request_async(
            method="GET",
            path="/v3/users/roles",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_getUserRoles",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.UsersGetUserRolesResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def remove_from_org(
        self,
        *,
        user_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> bytes:
        r"""Remove User From Org

        Remove user from organization. Upon sccess the user will be removed from the organization.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param user_id: (Required) user id
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UsersRemoveUserFromOrgRequest(
            user_id=user_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/v3/users/{userID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_removeUserFromOrg",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*/*"):
            http_res_bytes = utils.stream_to_bytes(http_res)
            return http_res_bytes
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def remove_from_org_async(
        self,
        *,
        user_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> bytes:
        r"""Remove User From Org

        Remove user from organization. Upon sccess the user will be removed from the organization.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param user_id: (Required) user id
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UsersRemoveUserFromOrgRequest(
            user_id=user_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v3/users/{userID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_removeUserFromOrg",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*/*"):
            http_res_bytes = await utils.stream_to_bytes_async(http_res)
            return http_res_bytes
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def get_by_id(
        self,
        *,
        user_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3UsersUserResponse:
        r"""Get User By ID

        Returns a users details of the given `userID` in the request param.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `read` scope.

        :param user_id: (Required) user id
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UsersGetUserByIDRequest(
            user_id=user_id,
        )

        req = self._build_request(
            method="GET",
            path="/v3/users/{userID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_getUserById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.UsersGetUserByIDResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def get_by_id_async(
        self,
        *,
        user_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3UsersUserResponse:
        r"""Get User By ID

        Returns a users details of the given `userID` in the request param.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `read` scope.

        :param user_id: (Required) user id
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UsersGetUserByIDRequest(
            user_id=user_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v3/users/{userID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_getUserById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.UsersGetUserByIDResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def update_by_id(
        self,
        *,
        user_id: str,
        role: models.V3UsersUpdateUserRequestRole,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3UsersUserResponse:
        r"""Update User by userID

        Update User by userID.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param user_id: (Required) user id
        :param role:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UsersUpdateUserByIDRequest(
            user_id=user_id,
            v3_users_update_user_request=models.V3UsersUpdateUserRequest(
                role=role,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v3/users/{userID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v3_users_update_user_request,
                False,
                False,
                "json",
                models.V3UsersUpdateUserRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_updateUserByID",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.UsersUpdateUserByIDResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def update_by_id_async(
        self,
        *,
        user_id: str,
        role: models.V3UsersUpdateUserRequestRole,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3UsersUserResponse:
        r"""Update User by userID

        Update User by userID.
        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `user-write` scope.

        :param user_id: (Required) user id
        :param role:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UsersUpdateUserByIDRequest(
            user_id=user_id,
            v3_users_update_user_request=models.V3UsersUpdateUserRequest(
                role=role,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v3/users/{userID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v3_users_update_user_request,
                False,
                False,
                "json",
                models.V3UsersUpdateUserRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Users_updateUserByID",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.UsersUpdateUserByIDResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3ServicesEscalationPolicyBasicTypedDict(TypedDict):
    r"""Represents the basic details of an Escalation Policy for a service."""

    id: str
    r"""The unique identifier for the escalation policy."""
    name: str
    r"""The name of the escalation policy."""
    description: str
    r"""A description of the escalation policy."""
    slug: str
    r"""The URL-friendly slug for the policy name."""


class V3ServicesEscalationPolicyBasic(BaseModel):
    r"""Represents the basic details of an Escalation Policy for a service."""

    id: str
    r"""The unique identifier for the escalation policy."""

    name: str
    r"""The name of the escalation policy."""

    description: str
    r"""A description of the escalation policy."""

    slug: str
    r"""The URL-friendly slug for the policy name."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4StatusPagesComponentsCreateComponentResponseTypedDict(TypedDict):
    id: int
    page_id: int
    name: str
    description: str
    under_maintenance: bool
    status_id: NotRequired[int]
    group_id: NotRequired[int]


class V4StatusPagesComponentsCreateComponentResponse(BaseModel):
    id: int

    page_id: Annotated[int, pydantic.Field(alias="pageID")]

    name: str

    description: str

    under_maintenance: Annotated[bool, pydantic.Field(alias="underMaintenance")]

    status_id: Annotated[Optional[int], pydantic.Field(alias="statusID")] = None

    group_id: Annotated[Optional[int], pydantic.Field(alias="groupID")] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V4StatusPagesIssuesIssueComponentTypedDict(TypedDict):
    id: NotRequired[int]
    name: NotRequired[str]


class V4StatusPagesIssuesIssueComponent(BaseModel):
    id: Optional[int] = None

    name: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_participantgroup import V4ParticipantGroup, V4ParticipantGroupTypedDict
from .v4_shifttimeslot import V4ShiftTimeSlot, V4ShiftTimeSlotTypedDict
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4CreateRotationRequestTypedDict(TypedDict):
    name: str
    start_date: str
    period: str
    change_participants_frequency: int
    change_participants_unit: str
    participant_groups: List[V4ParticipantGroupTypedDict]
    color: NotRequired[str]
    custom_period_frequency: NotRequired[int]
    custom_period_unit: NotRequired[str]
    shift_time_slots: NotRequired[List[V4ShiftTimeSlotTypedDict]]
    end_date: NotRequired[str]
    ends_after_iterations: NotRequired[int]


class V4CreateRotationRequest(BaseModel):
    name: str

    start_date: Annotated[str, pydantic.Field(alias="startDate")]

    period: str

    change_participants_frequency: Annotated[
        int, pydantic.Field(alias="changeParticipantsFrequency")
    ]

    change_participants_unit: Annotated[
        str, pydantic.Field(alias="changeParticipantsUnit")
    ]

    participant_groups: Annotated[
        List[V4ParticipantGroup], pydantic.Field(alias="participantGroups")
    ]

    color: Optional[str] = None

    custom_period_frequency: Annotated[
        Optional[int], pydantic.Field(alias="customPeriodFrequency")
    ] = None

    custom_period_unit: Annotated[
        Optional[str], pydantic.Field(alias="customPeriodUnit")
    ] = None

    shift_time_slots: Annotated[
        Optional[List[V4ShiftTimeSlot]], pydantic.Field(alias="shiftTimeSlots")
    ] = None

    end_date: Annotated[Optional[str], pydantic.Field(alias="endDate")] = None

    ends_after_iterations: Annotated[
        Optional[int], pydantic.Field(alias="endsAfterIterations")
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_runbooks_runbookresponse import (
    V3RunbooksRunbookResponse,
    V3RunbooksRunbookResponseTypedDict,
)
from .v3_runbooks_updaterunbookrequest import (
    V3RunbooksUpdateRunbookRequest,
    V3RunbooksUpdateRunbookRequestTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class RunbooksUpdateRunbookRequestTypedDict(TypedDict):
    runbook_id: str
    v3_runbooks_update_runbook_request: V3RunbooksUpdateRunbookRequestTypedDict


class RunbooksUpdateRunbookRequest(BaseModel):
    runbook_id: Annotated[
        str,
        pydantic.Field(alias="runbookID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v3_runbooks_update_runbook_request: Annotated[
        V3RunbooksUpdateRunbookRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class RunbooksUpdateRunbookResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3RunbooksRunbookResponseTypedDict


class RunbooksUpdateRunbookResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3RunbooksRunbookResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class WorkflowsBulkEnabledisableWorkflowsResponseBodyTypedDict(TypedDict):
    r"""There is no content to send for this request, but the headers may be useful."""


class WorkflowsBulkEnabledisableWorkflowsResponseBody(BaseModel):
    r"""There is no content to send for this request, but the headers may be useful."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3IncidentsIncidentActionsRebuildCircleCIProjectRequestTypedDict(TypedDict):
    r"""Request body for rebuilding a project in CircleCI."""

    vcs_type: str
    username: str
    reponame: str


class V3IncidentsIncidentActionsRebuildCircleCIProjectRequest(BaseModel):
    r"""Request body for rebuilding a project in CircleCI."""

    vcs_type: str

    username: str

    reponame: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_suppressionrules_customrepetition import (
    V3ServicesSuppressionRulesCustomRepetition,
    V3ServicesSuppressionRulesCustomRepetitionTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3ServicesSuppressionRulesTimeslotTypedDict(TypedDict):
    time_zone: str
    start_time: str
    end_time: str
    is_allday: bool
    repetition: str
    is_custom: bool
    custom: V3ServicesSuppressionRulesCustomRepetitionTypedDict
    ends_never: bool
    ends_on: str


class V3ServicesSuppressionRulesTimeslot(BaseModel):
    time_zone: str

    start_time: str

    end_time: str

    is_allday: bool

    repetition: str

    is_custom: bool

    custom: V3ServicesSuppressionRulesCustomRepetition

    ends_never: bool

    ends_on: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V4StatusPagesNewStatusPageThemeColorTypedDict(TypedDict):
    primary: NotRequired[str]
    secondary: NotRequired[str]


class V4StatusPagesNewStatusPageThemeColor(BaseModel):
    primary: Optional[str] = None

    secondary: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_squads_updatesquadnamerequest import (
    V4SquadsUpdateSquadNameRequest,
    V4SquadsUpdateSquadNameRequestTypedDict,
)
from .v4_squads_updatesquadnameresponse import (
    V4SquadsUpdateSquadNameResponse,
    V4SquadsUpdateSquadNameResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class SquadsUpdateSquadNameRequestTypedDict(TypedDict):
    squad_id: str
    v4_squads_update_squad_name_request: V4SquadsUpdateSquadNameRequestTypedDict


class SquadsUpdateSquadNameRequest(BaseModel):
    squad_id: Annotated[
        str,
        pydantic.Field(alias="squadID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v4_squads_update_squad_name_request: Annotated[
        V4SquadsUpdateSquadNameRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class SquadsUpdateSquadNameResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V4SquadsUpdateSquadNameResponseTypedDict


class SquadsUpdateSquadNameResponse(BaseModel):
    r"""The request has succeeded."""

    data: V4SquadsUpdateSquadNameResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_export_exportresponse import (
    V3ExportExportResponse,
    V3ExportExportResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class ExportGetExportDetailsRequestTypedDict(TypedDict):
    export_id: str


class ExportGetExportDetailsRequest(BaseModel):
    export_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]


class ExportGetExportDetailsResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3ExportExportResponseTypedDict


class ExportGetExportDetailsResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3ExportExportResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaloncallreminderrules_globaloncallreminderruleresponse import (
    V3GlobalOncallReminderRulesGlobalOncallReminderRuleResponse,
    V3GlobalOncallReminderRulesGlobalOncallReminderRuleResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing_extensions import Annotated, TypedDict


class GlobalOncallReminderRulesGetGlobalOncallReminderRulesRequestTypedDict(TypedDict):
    owner_id: str
    r"""required *"""


class GlobalOncallReminderRulesGetGlobalOncallReminderRulesRequest(BaseModel):
    owner_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""required *"""


class GlobalOncallReminderRulesGetGlobalOncallReminderRulesResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3GlobalOncallReminderRulesGlobalOncallReminderRuleResponseTypedDict


class GlobalOncallReminderRulesGetGlobalOncallReminderRulesResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3GlobalOncallReminderRulesGlobalOncallReminderRuleResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_auditlogs_actor import V3AuditLogsActor, V3AuditLogsActorTypedDict
from .v3_auditlogs_team import V3AuditLogsTeam, V3AuditLogsTeamTypedDict
import pydantic
from pydantic import model_serializer
from squadcast_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing_extensions import Annotated, NotRequired, TypedDict


class V3AuditLogsAuditLogIDResponseMetaTypedDict(TypedDict):
    pass


class V3AuditLogsAuditLogIDResponseMeta(BaseModel):
    pass


class V3AuditLogsAuditLogIDResponseTypedDict(TypedDict):
    r"""Represents detailed audit log entry response"""

    id: int
    resource: str
    action: str
    actor: V3AuditLogsActorTypedDict
    r"""Represents an actor (user) in audit logs"""
    client: str
    timestamp: str
    timezone: str
    team: V3AuditLogsTeamTypedDict
    r"""Represents a team in audit logs"""
    token_type: str
    ip_address: str
    additional_info: str
    user_agent: str
    meta: NotRequired[Nullable[V3AuditLogsAuditLogIDResponseMetaTypedDict]]


class V3AuditLogsAuditLogIDResponse(BaseModel):
    r"""Represents detailed audit log entry response"""

    id: int

    resource: str

    action: str

    actor: V3AuditLogsActor
    r"""Represents an actor (user) in audit logs"""

    client: str

    timestamp: str

    timezone: str

    team: V3AuditLogsTeam
    r"""Represents a team in audit logs"""

    token_type: Annotated[str, pydantic.Field(alias="tokenType")]

    ip_address: Annotated[str, pydantic.Field(alias="ipAddress")]

    additional_info: Annotated[str, pydantic.Field(alias="additionalInfo")]

    user_agent: Annotated[str, pydantic.Field(alias="userAgent")]

    meta: OptionalNullable[V3AuditLogsAuditLogIDResponseMeta] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["meta"]
        nullable_fields = ["meta"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


V3WorkflowsSqMakeHTTPCallUpdateName = Literal["sq_make_http_call",]


V3WorkflowsSqMakeHTTPCallUpdateMethod = Literal[
    "GET",
    "POST",
    "PUT",
    "PATCH",
    "DELETE",
]


class V3WorkflowsSqMakeHTTPCallUpdateHeaderTypedDict(TypedDict):
    key: str
    value: str


class V3WorkflowsSqMakeHTTPCallUpdateHeader(BaseModel):
    key: str

    value: str


class V3WorkflowsSqMakeHTTPCallUpdateDataTypedDict(TypedDict):
    method: NotRequired[V3WorkflowsSqMakeHTTPCallUpdateMethod]
    url: NotRequired[str]
    headers: NotRequired[List[V3WorkflowsSqMakeHTTPCallUpdateHeaderTypedDict]]
    body: NotRequired[str]


class V3WorkflowsSqMakeHTTPCallUpdateData(BaseModel):
    method: Optional[V3WorkflowsSqMakeHTTPCallUpdateMethod] = None

    url: Optional[str] = None

    headers: Optional[List[V3WorkflowsSqMakeHTTPCallUpdateHeader]] = None

    body: Optional[str] = None


class V3WorkflowsSqMakeHTTPCallUpdateTypedDict(TypedDict):
    name: V3WorkflowsSqMakeHTTPCallUpdateName
    data: NotRequired[V3WorkflowsSqMakeHTTPCallUpdateDataTypedDict]


class V3WorkflowsSqMakeHTTPCallUpdate(BaseModel):
    name: V3WorkflowsSqMakeHTTPCallUpdateName

    data: Optional[V3WorkflowsSqMakeHTTPCallUpdateData] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3RunbooksStepTypedDict(TypedDict):
    r"""Represents a single step in a runbook."""

    content: str
    r"""The content of the step, in Markdown format."""


class V3RunbooksStep(BaseModel):
    r"""Represents a single step in a runbook."""

    content: str
    r"""The content of the step, in Markdown format."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_routingrules_routingrule import (
    V3ServicesRoutingRulesRoutingRule,
    V3ServicesRoutingRulesRoutingRuleTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3ServicesRoutingRulesCreateOrUpdateRoutingRulesRequestTypedDict(TypedDict):
    rules: List[V3ServicesRoutingRulesRoutingRuleTypedDict]


class V3ServicesRoutingRulesCreateOrUpdateRoutingRulesRequest(BaseModel):
    rules: List[V3ServicesRoutingRulesRoutingRule]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_rulesetruleresponse import (
    V3GlobalEventRulesRulesetRuleResponse,
    V3GlobalEventRulesRulesetRuleResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from typing import Callable, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GlobalEventRulesListRulesetRulesRequestTypedDict(TypedDict):
    ger_id: int
    alert_source_version: str
    alert_source_shortname: str
    page_size: NotRequired[int]
    page_number: NotRequired[int]
    filters_search: NotRequired[str]


class GlobalEventRulesListRulesetRulesRequest(BaseModel):
    ger_id: Annotated[
        int, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    alert_source_version: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    alert_source_shortname: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    page_size: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    page_number: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    filters_search: Annotated[
        Optional[str],
        pydantic.Field(alias="filters.search"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None


class GlobalEventRulesListRulesetRulesMetaTypedDict(TypedDict):
    total_count: int


class GlobalEventRulesListRulesetRulesMeta(BaseModel):
    total_count: int


class GlobalEventRulesListRulesetRulesResponseBodyTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: List[V3GlobalEventRulesRulesetRuleResponseTypedDict]
    meta: GlobalEventRulesListRulesetRulesMetaTypedDict


class GlobalEventRulesListRulesetRulesResponseBody(BaseModel):
    r"""The request has succeeded."""

    data: List[V3GlobalEventRulesRulesetRuleResponse]

    meta: GlobalEventRulesListRulesetRulesMeta


class GlobalEventRulesListRulesetRulesResponseTypedDict(TypedDict):
    result: GlobalEventRulesListRulesetRulesResponseBodyTypedDict


class GlobalEventRulesListRulesetRulesResponse(BaseModel):
    next: Callable[[], Optional[GlobalEventRulesListRulesetRulesResponse]]

    result: GlobalEventRulesListRulesetRulesResponseBody



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class OwnerTypedDict(TypedDict):
    id: str
    type: str


class Owner(BaseModel):
    id: str

    type: str


class V3SquadsSquadResponseOrganizationTypedDict(TypedDict):
    id: str
    name: str
    slug: str


class V3SquadsSquadResponseOrganization(BaseModel):
    id: str

    name: str

    slug: str


class V3SquadsSquadResponseTypedDict(TypedDict):
    id: str
    name: str
    slug: str
    members: List[str]
    organization_id: str
    owner: OwnerTypedDict
    organization: V3SquadsSquadResponseOrganizationTypedDict


class V3SquadsSquadResponse(BaseModel):
    id: str

    name: str

    slug: str

    members: List[str]

    organization_id: str

    owner: Owner

    organization: V3SquadsSquadResponseOrganization



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_dependencies_createorupdatedependenciesrequest import (
    V3ServicesDependenciesCreateOrUpdateDependenciesRequest,
    V3ServicesDependenciesCreateOrUpdateDependenciesRequestTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class DependenciesCreateOrUpdateDependenciesRequestTypedDict(TypedDict):
    service_id: str
    v3_services_dependencies_create_or_update_dependencies_request: (
        V3ServicesDependenciesCreateOrUpdateDependenciesRequestTypedDict
    )


class DependenciesCreateOrUpdateDependenciesRequest(BaseModel):
    service_id: Annotated[
        str,
        pydantic.Field(alias="serviceID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v3_services_dependencies_create_or_update_dependencies_request: Annotated[
        V3ServicesDependenciesCreateOrUpdateDependenciesRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class DependenciesCreateOrUpdateDependenciesDataTypedDict(TypedDict):
    ok: bool


class DependenciesCreateOrUpdateDependenciesData(BaseModel):
    ok: bool


class DependenciesCreateOrUpdateDependenciesResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: DependenciesCreateOrUpdateDependenciesDataTypedDict


class DependenciesCreateOrUpdateDependenciesResponse(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: DependenciesCreateOrUpdateDependenciesData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WebhooksGetAllWebhooksRequestTypedDict(TypedDict):
    page_limit: NotRequired[str]
    offset: NotRequired[str]


class WebhooksGetAllWebhooksRequest(BaseModel):
    page_limit: Annotated[
        Optional[str],
        pydantic.Field(alias="page-limit"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    offset: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


V3EscalationPoliciesEscalationEntityType = Literal[
    "user",
    "squad",
    "schedule",
    "schedulev2",
]
r"""The type of the entity."""


class V3EscalationPoliciesEscalationEntityTypedDict(TypedDict):
    r"""Represents an entity to be notified in an escalation rule."""

    type: V3EscalationPoliciesEscalationEntityType
    r"""The type of the entity."""
    id: NotRequired[str]
    r"""The unique identifier of the entity (user, squad, or schedule v1)."""
    pid: NotRequired[int]
    r"""The unique identifier of the entity (schedule v2)."""


class V3EscalationPoliciesEscalationEntity(BaseModel):
    r"""Represents an entity to be notified in an escalation rule."""

    type: V3EscalationPoliciesEscalationEntityType
    r"""The type of the entity."""

    id: Optional[str] = None
    r"""The unique identifier of the entity (user, squad, or schedule v1)."""

    pid: Optional[int] = None
    r"""The unique identifier of the entity (schedule v2)."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V3ExportExportResponseTypedDict(TypedDict):
    id: str
    type: str
    status: str
    format_: str
    download_url: NotRequired[str]
    download_url_expires_at: NotRequired[str]
    error_message: NotRequired[str]


class V3ExportExportResponse(BaseModel):
    id: str

    type: str

    status: str

    format_: Annotated[str, pydantic.Field(alias="format")]

    download_url: Optional[str] = None

    download_url_expires_at: Optional[str] = None

    error_message: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4StatusPagesComponentGroupsCreateComponentGroupResponseTypedDict(TypedDict):
    id: int
    page_id: int
    name: str
    allow_subscription: bool
    description: NotRequired[str]


class V4StatusPagesComponentGroupsCreateComponentGroupResponse(BaseModel):
    id: int

    page_id: Annotated[int, pydantic.Field(alias="pageID")]

    name: str

    allow_subscription: Annotated[bool, pydantic.Field(alias="allowSubscription")]

    description: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_rotationresponse import V4RotationResponse, V4RotationResponseTypedDict
from .v4_updaterotationrequest import (
    V4UpdateRotationRequest,
    V4UpdateRotationRequestTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class RotationsUpdateRotationRequestTypedDict(TypedDict):
    schedule_id: str
    rotation_id: str
    v4_update_rotation_request: V4UpdateRotationRequestTypedDict


class RotationsUpdateRotationRequest(BaseModel):
    schedule_id: Annotated[
        str,
        pydantic.Field(alias="scheduleID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    rotation_id: Annotated[
        str,
        pydantic.Field(alias="rotationID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v4_update_rotation_request: Annotated[
        V4UpdateRotationRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class RotationsUpdateRotationResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V4RotationResponseTypedDict


class RotationsUpdateRotationResponse(BaseModel):
    r"""The request has succeeded."""

    data: V4RotationResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_icallinkresponse import V4ICalLinkResponse, V4ICalLinkResponseTypedDict
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from typing_extensions import Annotated, TypedDict


class ExportGetScheduleIcalLinkRequestTypedDict(TypedDict):
    schedule_id: str
    my_on_call: bool


class ExportGetScheduleIcalLinkRequest(BaseModel):
    schedule_id: Annotated[
        str,
        pydantic.Field(alias="scheduleID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    my_on_call: Annotated[
        bool,
        pydantic.Field(alias="myOnCall"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]


class ExportGetScheduleIcalLinkResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V4ICalLinkResponseTypedDict


class ExportGetScheduleIcalLinkResponse(BaseModel):
    r"""The request has succeeded."""

    data: V4ICalLinkResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_statuspages_components_component import (
    V4StatusPagesComponentsComponent,
    V4StatusPagesComponentsComponentTypedDict,
)
from .v4_statuspages_components_componentstatus import (
    V4StatusPagesComponentsComponentStatus,
    V4StatusPagesComponentsComponentStatusTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4StatusPagesComponentsListComponentsResponseTypedDict(TypedDict):
    id: NotRequired[int]
    type: NotRequired[str]
    name: NotRequired[str]
    allow_subscription: NotRequired[bool]
    description: NotRequired[str]
    status: NotRequired[V4StatusPagesComponentsComponentStatusTypedDict]
    under_maintenance: NotRequired[bool]
    status_maintenance: NotRequired[V4StatusPagesComponentsComponentStatusTypedDict]
    components: NotRequired[List[V4StatusPagesComponentsComponentTypedDict]]
    is_hidden: NotRequired[bool]


class V4StatusPagesComponentsListComponentsResponse(BaseModel):
    id: Optional[int] = None

    type: Optional[str] = None

    name: Optional[str] = None

    allow_subscription: Annotated[
        Optional[bool], pydantic.Field(alias="allowSubscription")
    ] = None

    description: Optional[str] = None

    status: Optional[V4StatusPagesComponentsComponentStatus] = None

    under_maintenance: Annotated[
        Optional[bool], pydantic.Field(alias="underMaintenance")
    ] = None

    status_maintenance: Annotated[
        Optional[V4StatusPagesComponentsComponentStatus],
        pydantic.Field(alias="statusMaintenance"),
    ] = None

    components: Optional[List[V4StatusPagesComponentsComponent]] = None

    is_hidden: Annotated[Optional[bool], pydantic.Field(alias="isHidden")] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V4TagTypedDict(TypedDict):
    key: str
    value: str
    color: str


class V4Tag(BaseModel):
    key: str

    value: str

    color: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_squads_squadmember import V4SquadsSquadMember, V4SquadsSquadMemberTypedDict
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V4SquadsCreateSquadResponseTypedDict(TypedDict):
    organization_id: str
    owner_id: str
    id: str
    name: str
    slug: str
    members: List[V4SquadsSquadMemberTypedDict]
    created_at: str
    created_by: str


class V4SquadsCreateSquadResponse(BaseModel):
    organization_id: str

    owner_id: str

    id: str

    name: str

    slug: str

    members: List[V4SquadsSquadMember]

    created_at: str

    created_by: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_participantgroup import V4ParticipantGroup, V4ParticipantGroupTypedDict
from .v4_shifttimeslot import V4ShiftTimeSlot, V4ShiftTimeSlotTypedDict
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4RotationResponseTypedDict(TypedDict):
    id: int
    name: str
    schedule_id: int
    color: str
    start_date: str
    period: str
    change_participants_frequency: int
    change_participants_unit: str
    created_at: str
    updated_at: str
    custom_period_frequency: NotRequired[int]
    custom_period_unit: NotRequired[str]
    shift_time_slots: NotRequired[List[V4ShiftTimeSlotTypedDict]]
    end_date: NotRequired[str]
    ends_after_iterations: NotRequired[int]
    rotation_participants: NotRequired[List[V4ParticipantGroupTypedDict]]


class V4RotationResponse(BaseModel):
    id: int

    name: str

    schedule_id: Annotated[int, pydantic.Field(alias="scheduleID")]

    color: str

    start_date: Annotated[str, pydantic.Field(alias="startDate")]

    period: str

    change_participants_frequency: Annotated[
        int, pydantic.Field(alias="changeParticipantsFrequency")
    ]

    change_participants_unit: Annotated[
        str, pydantic.Field(alias="changeParticipantsUnit")
    ]

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]

    custom_period_frequency: Annotated[
        Optional[int], pydantic.Field(alias="customPeriodFrequency")
    ] = None

    custom_period_unit: Annotated[
        Optional[str], pydantic.Field(alias="customPeriodUnit")
    ] = None

    shift_time_slots: Annotated[
        Optional[List[V4ShiftTimeSlot]], pydantic.Field(alias="shiftTimeSlots")
    ] = None

    end_date: Annotated[Optional[str], pydantic.Field(alias="endDate")] = None

    ends_after_iterations: Annotated[
        Optional[int], pydantic.Field(alias="endsAfterIterations")
    ] = None

    rotation_participants: Annotated[
        Optional[List[V4ParticipantGroup]], pydantic.Field(alias="rotationParticipants")
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_auditlogs_auditlogresponse import (
    V3AuditLogsAuditLogResponse,
    V3AuditLogsAuditLogResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import Annotated, TypedDict


class V3AuditLogsListAuditLogsResponseMetadataTypedDict(TypedDict):
    total_count: int


class V3AuditLogsListAuditLogsResponseMetadata(BaseModel):
    total_count: Annotated[int, pydantic.Field(alias="totalCount")]


class V3AuditLogsListAuditLogsResponseTypedDict(TypedDict):
    r"""Response model for listing audit logs"""

    data: List[V3AuditLogsAuditLogResponseTypedDict]
    metadata: V3AuditLogsListAuditLogsResponseMetadataTypedDict


class V3AuditLogsListAuditLogsResponse(BaseModel):
    r"""Response model for listing audit logs"""

    data: List[V3AuditLogsAuditLogResponse]

    metadata: V3AuditLogsListAuditLogsResponseMetadata



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_runbooks_runbookresponse import (
    V3RunbooksRunbookResponse,
    V3RunbooksRunbookResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class RunbooksGetAllRunbooksByTeamResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: List[V3RunbooksRunbookResponseTypedDict]


class RunbooksGetAllRunbooksByTeamResponse(BaseModel):
    r"""The request has succeeded."""

    data: List[V3RunbooksRunbookResponse]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_suppressionrules_suppressionrule import (
    V3ServicesSuppressionRulesSuppressionRule,
    V3ServicesSuppressionRulesSuppressionRuleTypedDict,
)
from datetime import datetime
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3ServicesSuppressionRulesSuppressionRulesContainerResponseTypedDict(TypedDict):
    id: str
    created_at: datetime
    updated_at: datetime
    service_id: str
    organization_id: str
    rules: List[V3ServicesSuppressionRulesSuppressionRuleTypedDict]


class V3ServicesSuppressionRulesSuppressionRulesContainerResponse(BaseModel):
    id: str

    created_at: datetime

    updated_at: datetime

    service_id: str

    organization_id: str

    rules: List[V3ServicesSuppressionRulesSuppressionRule]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_snoozenotifications_snoozeincidentrequest import (
    V3IncidentsSnoozeNotificationsSnoozeIncidentRequest,
    V3IncidentsSnoozeNotificationsSnoozeIncidentRequestTypedDict,
)
from .v3_incidents_snoozenotifications_snoozeincidentresponse import (
    V3IncidentsSnoozeNotificationsSnoozeIncidentResponse,
    V3IncidentsSnoozeNotificationsSnoozeIncidentResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class SnoozeNotificationsSnoozeIncidentNotificationsRequestTypedDict(TypedDict):
    incident_id: str
    v3_incidents_snooze_notifications_snooze_incident_request: (
        V3IncidentsSnoozeNotificationsSnoozeIncidentRequestTypedDict
    )


class SnoozeNotificationsSnoozeIncidentNotificationsRequest(BaseModel):
    incident_id: Annotated[
        str,
        pydantic.Field(alias="incidentID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v3_incidents_snooze_notifications_snooze_incident_request: Annotated[
        V3IncidentsSnoozeNotificationsSnoozeIncidentRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class SnoozeNotificationsSnoozeIncidentNotificationsResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3IncidentsSnoozeNotificationsSnoozeIncidentResponseTypedDict


class SnoozeNotificationsSnoozeIncidentNotificationsResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3IncidentsSnoozeNotificationsSnoozeIncidentResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4StatusPagesMaintenancesCreateMaintenanceResponseTypedDict(TypedDict):
    id: int
    title: str
    page_id: int
    note: str
    start_time: datetime
    end_time: datetime
    components: NotRequired[List[int]]


class V4StatusPagesMaintenancesCreateMaintenanceResponse(BaseModel):
    id: int

    title: str

    page_id: Annotated[int, pydantic.Field(alias="pageID")]

    note: str

    start_time: Annotated[datetime, pydantic.Field(alias="startTime")]

    end_time: Annotated[datetime, pydantic.Field(alias="endTime")]

    components: Optional[List[int]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_entityowner import (
    V3GlobalEventRulesEntityOwner,
    V3GlobalEventRulesEntityOwnerTypedDict,
)
from .v3_globaleventrules_ruleset import (
    V3GlobalEventRulesRuleset,
    V3GlobalEventRulesRulesetTypedDict,
)
from datetime import datetime
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3GlobalEventRulesGlobalEventRuleInListTypedDict(TypedDict):
    id: NotRequired[int]
    org_id: NotRequired[str]
    team_id: NotRequired[str]
    name: NotRequired[str]
    description: NotRequired[str]
    routing_key: NotRequired[str]
    entity_owner: NotRequired[V3GlobalEventRulesEntityOwnerTypedDict]
    rulesets: NotRequired[List[V3GlobalEventRulesRulesetTypedDict]]
    created_at: NotRequired[datetime]
    created_by: NotRequired[str]
    updated_at: NotRequired[datetime]
    updated_by: NotRequired[str]


class V3GlobalEventRulesGlobalEventRuleInList(BaseModel):
    id: Optional[int] = None

    org_id: Optional[str] = None

    team_id: Optional[str] = None

    name: Optional[str] = None

    description: Optional[str] = None

    routing_key: Optional[str] = None

    entity_owner: Optional[V3GlobalEventRulesEntityOwner] = None

    rulesets: Optional[List[V3GlobalEventRulesRuleset]] = None

    created_at: Optional[datetime] = None

    created_by: Optional[str] = None

    updated_at: Optional[datetime] = None

    updated_by: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


V3GlobalEventRulesEntityOwnerType = Literal[
    "team",
    "user",
    "squad",
]


class V3GlobalEventRulesEntityOwnerTypedDict(TypedDict):
    id: str
    type: V3GlobalEventRulesEntityOwnerType


class V3GlobalEventRulesEntityOwner(BaseModel):
    id: str

    type: V3GlobalEventRulesEntityOwnerType



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_extensions_msteams_squadcastservicemapping import (
    V3ExtensionsMSTeamsSquadCastServiceMapping,
    V3ExtensionsMSTeamsSquadCastServiceMappingTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3ExtensionsMSTeamsChannelConfigurationTypedDict(TypedDict):
    r"""Defines a mapping from a Squadcast Team/Service to a specific MS Teams channel."""

    squadcast_team_id: str
    r"""The MongoDB ObjectID of the corresponding team in Squadcast."""
    squadcast_team_name: str
    r"""The name of the corresponding team in Squadcast."""
    is_all_services: bool
    r"""If true, alerts for all services in the Squadcast Team are sent to this channel."""
    services: List[V3ExtensionsMSTeamsSquadCastServiceMappingTypedDict]
    r"""An array of specific services to route to this channel. Used when 'is_all_services' is false."""
    msteams_channel_id: str
    r"""The unique identifier of the target channel in MS Teams."""
    msteams_channel_name: str
    r"""The display name of the target channel in MS Teams."""
    id: NotRequired[str]
    r"""The MongoDB ObjectID for this specific channel configuration entry."""


class V3ExtensionsMSTeamsChannelConfiguration(BaseModel):
    r"""Defines a mapping from a Squadcast Team/Service to a specific MS Teams channel."""

    squadcast_team_id: str
    r"""The MongoDB ObjectID of the corresponding team in Squadcast."""

    squadcast_team_name: str
    r"""The name of the corresponding team in Squadcast."""

    is_all_services: bool
    r"""If true, alerts for all services in the Squadcast Team are sent to this channel."""

    services: List[V3ExtensionsMSTeamsSquadCastServiceMapping]
    r"""An array of specific services to route to this channel. Used when 'is_all_services' is false."""

    msteams_channel_id: str
    r"""The unique identifier of the target channel in MS Teams."""

    msteams_channel_name: str
    r"""The display name of the target channel in MS Teams."""

    id: Optional[str] = None
    r"""The MongoDB ObjectID for this specific channel configuration entry."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_webforms_recaptchasecrets import (
    V3WebformsRecaptchaSecrets,
    V3WebformsRecaptchaSecretsTypedDict,
)
from .v3_webforms_wfinputfield import (
    V3WebformsWFInputField,
    V3WebformsWFInputFieldTypedDict,
)
from .v3_webforms_wfservice import V3WebformsWFService, V3WebformsWFServiceTypedDict
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3WebformsCreateOrUpdateWebformRequestTagsTypedDict(TypedDict):
    r"""Key-value tags for the webform"""


class V3WebformsCreateOrUpdateWebformRequestTags(BaseModel):
    r"""Key-value tags for the webform"""


class V3WebformsCreateOrUpdateWebformRequestTypedDict(TypedDict):
    r"""Public API request for creating a webform"""

    owner_id: str
    r"""Unique identifier of the owner (string or UUID)"""
    name: str
    r"""Name of the webform"""
    is_cname: bool
    r"""Indicates if a custom domain (CNAME) is used"""
    is_captcha_enabled: bool
    r"""Whether CAPTCHA is enabled on the form"""
    captcha_secret: V3WebformsRecaptchaSecretsTypedDict
    r"""CAPTCHA credentials to be validated"""
    form_owner_type: str
    r"""Entity type that owns the form (e.g., team, user)"""
    form_owner_id: str
    r"""Identifier for the owner entity"""
    services: List[V3WebformsWFServiceTypedDict]
    r"""List of services tied to this webform"""
    header: str
    r"""Header text shown on the form"""
    title: str
    r"""Title of the webform"""
    footer_text: str
    r"""Text displayed in the footer"""
    footer_link: str
    r"""Hyperlink in the footer (mailto or https)"""
    host_name: NotRequired[str]
    r"""Custom hostname if CNAME is enabled"""
    tags: NotRequired[V3WebformsCreateOrUpdateWebformRequestTagsTypedDict]
    r"""Key-value tags for the webform"""
    input_field: NotRequired[List[V3WebformsWFInputFieldTypedDict]]
    r"""Input fields to be rendered on the form"""
    logo_url: NotRequired[str]
    r"""URL to the organization's logo"""
    email_on: NotRequired[List[str]]
    r"""Emails to notify on submission"""
    description: NotRequired[str]
    r"""Optional description for the webform"""


class V3WebformsCreateOrUpdateWebformRequest(BaseModel):
    r"""Public API request for creating a webform"""

    owner_id: str
    r"""Unique identifier of the owner (string or UUID)"""

    name: str
    r"""Name of the webform"""

    is_cname: bool
    r"""Indicates if a custom domain (CNAME) is used"""

    is_captcha_enabled: bool
    r"""Whether CAPTCHA is enabled on the form"""

    captcha_secret: V3WebformsRecaptchaSecrets
    r"""CAPTCHA credentials to be validated"""

    form_owner_type: str
    r"""Entity type that owns the form (e.g., team, user)"""

    form_owner_id: str
    r"""Identifier for the owner entity"""

    services: List[V3WebformsWFService]
    r"""List of services tied to this webform"""

    header: str
    r"""Header text shown on the form"""

    title: str
    r"""Title of the webform"""

    footer_text: str
    r"""Text displayed in the footer"""

    footer_link: str
    r"""Hyperlink in the footer (mailto or https)"""

    host_name: Optional[str] = None
    r"""Custom hostname if CNAME is enabled"""

    tags: Optional[V3WebformsCreateOrUpdateWebformRequestTags] = None
    r"""Key-value tags for the webform"""

    input_field: Optional[List[V3WebformsWFInputField]] = None
    r"""Input fields to be rendered on the form"""

    logo_url: Optional[str] = None
    r"""URL to the organization's logo"""

    email_on: Optional[List[str]] = None
    r"""Emails to notify on submission"""

    description: Optional[str] = None
    r"""Optional description for the webform"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_serviceresponse import (
    V3ServicesServiceResponse,
    V3ServicesServiceResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ServicesGetServicesRequestTypedDict(TypedDict):
    owner_id: str
    entity_owner: NotRequired[str]
    name: NotRequired[str]


class ServicesGetServicesRequest(BaseModel):
    owner_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=False))
    ]

    entity_owner: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    name: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None


class ServicesGetServicesResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: List[V3ServicesServiceResponseTypedDict]


class ServicesGetServicesResponse(BaseModel):
    r"""The request has succeeded."""

    data: List[V3ServicesServiceResponse]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_escalationpolicies_escalationpolicyresponse import (
    V3EscalationPoliciesEscalationPolicyResponse,
    V3EscalationPoliciesEscalationPolicyResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class EscalationPoliciesGetEscalationPolicyByIDRequestTypedDict(TypedDict):
    escalation_policy_id: str
    r"""(Required) escalation policy ID"""


class EscalationPoliciesGetEscalationPolicyByIDRequest(BaseModel):
    escalation_policy_id: Annotated[
        str,
        pydantic.Field(alias="escalationPolicyID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""(Required) escalation policy ID"""


class EscalationPoliciesGetEscalationPolicyByIDResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3EscalationPoliciesEscalationPolicyResponseTypedDict
    r"""Represents an Escalation Policy in the system."""


class EscalationPoliciesGetEscalationPolicyByIDResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3EscalationPoliciesEscalationPolicyResponse
    r"""Represents an Escalation Policy in the system."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3WorkflowsUpdateActionsOrderRequestTypedDict(TypedDict):
    action_order: NotRequired[List[int]]


class V3WorkflowsUpdateActionsOrderRequest(BaseModel):
    action_order: Optional[List[int]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_additionalresponders_additionalresponder import (
    V3IncidentsAdditionalRespondersAdditionalResponder,
    V3IncidentsAdditionalRespondersAdditionalResponderTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3IncidentsAdditionalRespondersAdditionalResponderResponseTypedDict(TypedDict):
    additional_responders: List[
        V3IncidentsAdditionalRespondersAdditionalResponderTypedDict
    ]


class V3IncidentsAdditionalRespondersAdditionalResponderResponse(BaseModel):
    additional_responders: List[V3IncidentsAdditionalRespondersAdditionalResponder]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing_extensions import Annotated, TypedDict


class V4ChangeTimezoneResponseTypedDict(TypedDict):
    time_zone: str


class V4ChangeTimezoneResponse(BaseModel):
    time_zone: Annotated[str, pydantic.Field(alias="timeZone")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_postmortems_postmortemattachmentrequest import (
    V3IncidentsPostmortemsPostmortemAttachmentRequest,
    V3IncidentsPostmortemsPostmortemAttachmentRequestTypedDict,
)
from .v3_incidents_postmortems_postmortemfollowup import (
    V3IncidentsPostmortemsPostmortemFollowUp,
    V3IncidentsPostmortemsPostmortemFollowUpTypedDict,
)
from .v3_incidents_postmortems_postmortemstatus import (
    V3IncidentsPostmortemsPostmortemStatus,
)
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3IncidentsPostmortemsCreatePostmortemRequestTypedDict(TypedDict):
    r"""Request body for creating a new postmortem."""

    owner_id: str
    title: str
    postmortem: str
    status: V3IncidentsPostmortemsPostmortemStatus
    r"""Represents the status of a postmortem."""
    follow_ups: List[V3IncidentsPostmortemsPostmortemFollowUpTypedDict]
    attachments: List[V3IncidentsPostmortemsPostmortemAttachmentRequestTypedDict]


class V3IncidentsPostmortemsCreatePostmortemRequest(BaseModel):
    r"""Request body for creating a new postmortem."""

    owner_id: str

    title: str

    postmortem: str

    status: V3IncidentsPostmortemsPostmortemStatus
    r"""Represents the status of a postmortem."""

    follow_ups: List[V3IncidentsPostmortemsPostmortemFollowUp]

    attachments: List[V3IncidentsPostmortemsPostmortemAttachmentRequest]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_statuspages_componentgroups_component import (
    V4StatusPagesComponentGroupsComponent,
    V4StatusPagesComponentGroupsComponentTypedDict,
)
from .v4_statuspages_componentgroups_componentstatus import (
    V4StatusPagesComponentGroupsComponentStatus,
    V4StatusPagesComponentGroupsComponentStatusTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4StatusPagesComponentGroupsComponentGroupResponseTypedDict(TypedDict):
    id: int
    type: str
    name: str
    allow_subscription: bool
    description: NotRequired[str]
    status: NotRequired[V4StatusPagesComponentGroupsComponentStatusTypedDict]
    under_maintenance: NotRequired[bool]
    status_maintenance: NotRequired[
        V4StatusPagesComponentGroupsComponentStatusTypedDict
    ]
    components: NotRequired[List[V4StatusPagesComponentGroupsComponentTypedDict]]
    is_hidden: NotRequired[bool]


class V4StatusPagesComponentGroupsComponentGroupResponse(BaseModel):
    id: int

    type: str

    name: str

    allow_subscription: Annotated[bool, pydantic.Field(alias="allowSubscription")]

    description: Optional[str] = None

    status: Optional[V4StatusPagesComponentGroupsComponentStatus] = None

    under_maintenance: Annotated[
        Optional[bool], pydantic.Field(alias="underMaintenance")
    ] = None

    status_maintenance: Annotated[
        Optional[V4StatusPagesComponentGroupsComponentStatus],
        pydantic.Field(alias="statusMaintenance"),
    ] = None

    components: Optional[List[V4StatusPagesComponentGroupsComponent]] = None

    is_hidden: Annotated[Optional[bool], pydantic.Field(alias="isHidden")] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3AuditLogsExportAuditLogsResponseDataTypedDict(TypedDict):
    id: str
    message: str


class V3AuditLogsExportAuditLogsResponseData(BaseModel):
    id: str

    message: str


class V3AuditLogsExportAuditLogsResponseTypedDict(TypedDict):
    r"""Response model for exporting audit logs"""

    data: V3AuditLogsExportAuditLogsResponseDataTypedDict


class V3AuditLogsExportAuditLogsResponse(BaseModel):
    r"""Response model for exporting audit logs"""

    data: V3AuditLogsExportAuditLogsResponseData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_incidentactions_webhookactionresponse import (
    V3IncidentsIncidentActionsWebhookActionResponse,
    V3IncidentsIncidentActionsWebhookActionResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class IncidentActionsTriggerAWebhookManuallyRequestTypedDict(TypedDict):
    incident_id: str
    event_webhook_id: str


class IncidentActionsTriggerAWebhookManuallyRequest(BaseModel):
    incident_id: Annotated[
        str,
        pydantic.Field(alias="incidentID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    event_webhook_id: Annotated[
        str,
        pydantic.Field(alias="eventWebhookID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]


class IncidentActionsTriggerAWebhookManuallyResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3IncidentsIncidentActionsWebhookActionResponseTypedDict
    r"""Represents the response structure for triggering a webhook manually."""


class IncidentActionsTriggerAWebhookManuallyResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3IncidentsIncidentActionsWebhookActionResponse
    r"""Represents the response structure for triggering a webhook manually."""
